@@ -45,7 +45,12 @@ insert()
 {
 /sbin/lsmod | grep psfreedom > /dev/null                         
 if [ $? != 0 ]; then
-    insmod /opt/psfreedom/modules/psfreedom.ko
+    if [ "$2" = "--jig" ]
+    then
+    	insmod /opt/psfreedom/modules/psfreedom.ko jig=1
+    else
+	insmod /opt/psfreedom/modules/psfreedom.ko
+    fi
     RC=$?                                    
 fi                                                                   
                                                             
@@ -58,7 +63,7 @@ fi
 argv0=$1 #Fucking BusyBox
 case "$argv0" in
 	--pl3)
-   		/usr/sbin/psfreedommoduledetection.sh --pl3
+	   	/usr/sbin/psfreedommoduledetection.sh --pl3
 		insert
 		;;
 
