@@ -18,6 +18,8 @@
 
 #include "config.h"
 
+#define DEBUG
+#define VERBOSE_DEBUG
 
 #include <linux/version.h>
 #include <linux/module.h>
@@ -39,23 +41,56 @@
 
 MODULE_AUTHOR("Youness Alaoui (KaKaRoTo)");
 MODULE_LICENSE("GPL");
+MODULE_DESCRIPTION("Implements PS3 jailbreak exploit over USB.");
 
 #define DRIVER_VERSION "19 October 2010"
 #define PSFREEDOM_VERSION "1.1"
 
-#define PROC_DIR_NAME		      "psfreedom"
+MODULE_VERSION(PSFREEDOM_VERSION);
+
+#define PROC_DIR_NAME                 "psfreedom"
 #define PROC_STATUS_NAME              "status"
 #define PROC_VERSION_NAME             "version"
 #define PROC_PAYLOAD_NAME             "payload"
 #define PROC_SHELLCODE_NAME           "shellcode"
 #define PROC_SUPPORTED_FIRMWARES_NAME "supported_firmwares"
 #define PROC_FW_VERSION_NAME          "fw_version"
+#define PROC_STAGE2_NAME              "asbestos_stage2"
 
 static const char shortname[] = "PSFreedom";
 static const char longname[] = "PS3 Jailbreak exploit";
 
+static int asbestos = 0;
+static int jig = 0;
+
+static short int debug = 0;
+
+#ifdef NO_DELAYED_PORT_SWITCHING
+static int no_delayed_switching = 1;
+#else
+static int no_delayed_switching = 0;
+#endif
+
+
+module_param(asbestos, bool, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
+MODULE_PARM_DESC(asbestos,
+   " Asbestos mode, connects device 6, use it after having the stage1 loaded.");
+
+module_param(jig, bool, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
+MODULE_PARM_DESC(jig,
+   " JIG mode, connects only the JIG dongle, use it to enable service mode.");
+
+module_param(debug, short, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
+MODULE_PARM_DESC(debug, " Debug level. (0=none, 1=normal, 2=verbose)");
+
+module_param(no_delayed_switching, bool,
+             S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP);
+MODULE_PARM_DESC(no_delayed_switching,
+                 " Enable no delayed port switching mode.");
+
 /* big enough to hold our biggest descriptor */
-#define USB_BUFSIZ 256
+#define MAXIMUM_CONFIG_DESCRIPTOR_SIZE 4096
+#define USB_BUFSIZ MAXIMUM_CONFIG_DESCRIPTOR_SIZE
 
 /* States for the state machine */
 enum PsfreedomState {
@@ -82,6 +117,8 @@ enum PsfreedomState {
   DEVICE5_READY,
   DEVICE5_WAIT_DISCONNECT,
   DEVICE5_DISCONNECTED,
+  DEVICE6_WAIT_READY,
+  DEVICE6_READY,
   DONE,
 };
 
@@ -109,6 +146,8 @@ enum PsfreedomState {
       s==DEVICE5_READY?"DEVICE5_READY":                         \
       s==DEVICE5_WAIT_DISCONNECT?"DEVICE5_WAIT_DISCONNECT":     \
       s==DEVICE5_DISCONNECTED?"DEVICE5_DISCONNECTED":           \
+      s==DEVICE6_WAIT_READY?"DEVICE6_WAIT_READY":               \
+      s==DEVICE6_READY?"DEVICE6_READY":                         \
       s==DONE?"DONE":                                           \
       "UNKNOWN_STATE")
 
@@ -121,6 +160,9 @@ enum PsfreedomState {
       r==0xa300?"GET_PORT_STATUS":              \
       r==0x2301?"CLEAR_PORT_FEATURE":           \
       r==0x010B?"SET_INTERFACE":                \
+      r==0x4001?"ASBESTOS_PRINT_DBG_MSG":       \
+      r==0xc002?"ASBESTOS_GET_STAGE2_SIZE":     \
+      r==0xc003?"ASBESTOS_READ_STAGE2_BLOCK":   \
       "UNKNOWN")
 
 #include "hub.h"
@@ -139,32 +181,32 @@ typedef struct {
 
 /* Out device structure */
 struct psfreedom_device {
-  spinlock_t		lock;
-  struct usb_gadget	*gadget;
+  spinlock_t            lock;
+  struct usb_gadget     *gadget;
   /* for control responses */
-  struct usb_request	*req;
+  struct usb_request    *req;
   /* for hub interrupts */
-  struct usb_request	*hub_req;
+  struct usb_request    *hub_req;
   /* The hub uses a non standard ep2in */
-  struct usb_ep		*hub_ep;
+  struct usb_ep         *hub_ep;
   /* BULK IN for the JIG */
-  struct usb_ep		*in_ep;
+  struct usb_ep         *in_ep;
   /* BULK OUT for the JIG */
-  struct usb_ep		*out_ep;
+  struct usb_ep         *out_ep;
   /* status of the state machine */
-  enum PsfreedomState	status;
+  enum PsfreedomState   status;
   /* The port to switch to after a delay */
-  int			switch_to_port_delayed;
+  int                   switch_to_port_delayed;
   /* Received length of the JIG challenge */
-  int			challenge_len;
+  int                   challenge_len;
   /* Sent length of the JIG response */
-  int			response_len;
+  int                   response_len;
   /* Hub port status/change */
-  struct hub_port	hub_ports[6];
+  struct hub_port       hub_ports[6];
   /* Currently enabled port on the hub (0 == hub) */
-  unsigned int		current_port;
+  unsigned int          current_port;
   /* The address of all ports (0 == hub) */
-  u8			port_address[7];
+  u8                    port_address[7];
   /* The port1 configuration descriptor. dynamically loaded from procfs */
   u8 *port1_config_desc;
   unsigned int port1_config_desc_size;
@@ -176,8 +218,21 @@ struct psfreedom_device {
   struct proc_dir_entry *proc_shellcode_entry;
   struct proc_dir_entry *proc_supported_firmwares_entry;
   struct proc_dir_entry *proc_fw_version_entry;
+  struct proc_dir_entry *proc_stage2_entry;
   /* current firmware compatibility */
-  const Firmware_t	*firmware;
+  const Firmware_t      *firmware;
+  /* pointer to stage2 payload */
+  char *stage2_payload;
+  unsigned int stage2_payload_size;
+  /* Module params */
+  /* For some *magical* reason, the first module parameter being referenced
+     in the DBG macro will get reset to its default value after the init proc
+     is called... so we save those values in here instead... wtf is happening..
+  */
+  int asbestos;
+  int jig;
+  short int debug;
+  int no_delayed_switching;
 };
 
 
@@ -202,12 +257,18 @@ struct psfreedom_device {
 #define ERROR(d, fmt, args...)                  \
   dev_err(&(d)->gadget->dev , fmt , ## args)
 
-#define DBG(d, fmt, args...)                    \
-  dev_dbg(&(d)->gadget->dev , fmt , ## args)
+#define DBG(d, fmt, args...)                        \
+  {                                                 \
+    if (d->debug > 0)                               \
+      dev_dbg(&(d)->gadget->dev , fmt , ## args);   \
+  }
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,23)
-#define VDBG(d, fmt, args...)                   \
-  dev_vdbg(&(d)->gadget->dev , fmt , ## args)
+#define VDBG(d, fmt, args...)                       \
+  {                                                 \
+    if (d->debug > 1)                               \
+      dev_vdbg(&(d)->gadget->dev , fmt , ## args);  \
+  }
 #else
 #define VDBG DBG
 #endif
@@ -273,20 +334,31 @@ static void psfreedom_state_machine_timeout(unsigned long data)
       jig_response_send (dev, NULL);
       break;
     case DEVICE5_READY:
-#ifdef NO_DELAYED_PORT_SWITCHING
-      /* if we can't delay the port switching, then we at this point, we can't
-         disconnect the device 3... so we just unregister the driver so that
-         all the devices get virtually disconnected and the exploit works.
-         Since we won't exist after that, let's unlock the spinlock and return.
-      */
-      spin_unlock_irqrestore (&dev->lock, flags);
-      dev->status = DONE;
-      psfreedom_cleanup ();
-      return;
-#else
-      dev->status = DEVICE3_WAIT_DISCONNECT;
-      hub_disconnect_port (dev, 3);
-#endif
+      if (dev->jig) {
+        dev->status = DONE;
+        INFO (dev, "JIG response sent. Should be authenticated now\n");
+        INFO (dev, "DONE!");
+        del_timer (&psfreedom_state_machine_timer);
+        timer_added = 0;
+      } else if (no_delayed_switching) {
+        /* if we can't delay the port switching, then we at this point, we can't
+          disconnect the device 3... so we just unregister the driver so that
+          all the devices get virtually disconnected and the exploit works.
+          Since we won't exist after that, let's unlock the spinlock and return.
+         */
+        INFO (dev, "JAILBROKEN!!! DONE!!!!!!!!!\n");
+        INFO (dev, "Congratulations, it should work now, "
+              "all you need to do is pray!");
+        del_timer (&psfreedom_state_machine_timer);
+        timer_added = 0;
+        dev->status = DONE;
+        spin_unlock_irqrestore (&dev->lock, flags);
+        psfreedom_cleanup ();
+        return;
+      } else {
+        dev->status = DEVICE3_WAIT_DISCONNECT;
+        hub_disconnect_port (dev, 3);
+      }
       break;
     case DEVICE3_DISCONNECTED:
       dev->status = DEVICE5_WAIT_DISCONNECT;
@@ -306,11 +378,17 @@ static void psfreedom_state_machine_timeout(unsigned long data)
       hub_disconnect_port (dev, 1);
       break;
     case DEVICE1_DISCONNECTED:
-      dev->status = DONE;
-      INFO (dev, "JAILBROKEN!!! DONE!!!!!!!!!\n");
-      INFO (dev, "Congratulations, worked!");
-      del_timer (&psfreedom_state_machine_timer);
-      timer_added = 0;
+      /* simple check to see if a stage2 is loaded */
+      if (dev->stage2_payload) {
+        dev->status = DEVICE6_WAIT_READY;
+        hub_connect_port (dev, 6);
+      } else {
+        dev->status = DONE;
+        INFO (dev, "JAILBROKEN!!! DONE!!!!!!!!!\n");
+        INFO (dev, "Congratulations, worked!");
+        del_timer (&psfreedom_state_machine_timer);
+        timer_added = 0;
+      }
       break;
     default:
       break;
@@ -321,7 +399,7 @@ static void psfreedom_state_machine_timeout(unsigned long data)
 
 static struct usb_request *alloc_ep_req(struct usb_ep *ep, unsigned length)
 {
-  struct usb_request	*req;
+  struct usb_request *req;
 
   req = usb_ep_alloc_request(ep, GFP_ATOMIC);
   if (req) {
@@ -353,7 +431,16 @@ static void psfreedom_disconnect (struct usb_gadget *gadget)
   /* Reinitialize all device variables*/
   dev->challenge_len = 0;
   dev->response_len = 0;
-  dev->current_port = 0;
+  if (dev->asbestos) {
+    dev->current_port = 6;
+    dev->status = DEVICE6_WAIT_READY;
+  } else if (dev->jig) {
+    dev->current_port = 5;
+    dev->status = DEVICE5_WAIT_READY;
+  } else {
+    dev->current_port = 0;
+    dev->status = INIT;
+  }
   for (i = 0; i < 6; i++)
     dev->hub_ports[i].status = dev->hub_ports[i].change = 0;
   for (i = 0; i < 7; i++)
@@ -364,7 +451,6 @@ static void psfreedom_disconnect (struct usb_gadget *gadget)
     del_timer (&psfreedom_state_machine_timer);
   timer_added = 0;
   dev->switch_to_port_delayed = -1;
-  dev->status = INIT;
 
   spin_unlock_irqrestore (&dev->lock, flags);
 }
@@ -407,7 +493,7 @@ static int psfreedom_setup(struct usb_gadget *gadget,
   u16 request = (ctrl->bRequestType << 8) | ctrl->bRequest;
 
   spin_lock_irqsave (&dev->lock, flags);
-  VDBG (dev, "Setup called %d (%d) -- %d -- %d. Myaddr :%d\n", ctrl->bRequest,
+  VDBG (dev, "Setup called %d (0x%x) -- %d -- %d. Myaddr :%d\n", ctrl->bRequest,
       ctrl->bRequestType, w_value, w_index, address);
 
   req->zero = 0;
@@ -427,11 +513,11 @@ static int psfreedom_setup(struct usb_gadget *gadget,
   else
     value = devices_setup (gadget, ctrl, request, w_index, w_value, w_length);
 
-#ifdef NO_DELAYED_PORT_SWITCHING
-  if (dev->switch_to_port_delayed >= 0)
-    switch_to_port (dev, dev->switch_to_port_delayed);
-  dev->switch_to_port_delayed = -1;
-#endif
+  if (no_delayed_switching) {
+    if (dev->switch_to_port_delayed >= 0)
+      switch_to_port (dev, dev->switch_to_port_delayed);
+    dev->switch_to_port_delayed = -1;
+  }
 
   DBG (dev, "%s Setup called %s (%d - %d) -> %d (w_length=%d)\n",
       STATUS_STR (dev->status),  REQUEST_STR (request), w_value, w_index,
@@ -529,7 +615,7 @@ int proc_payload_read(char *buffer, char **start, off_t offset, int count,
 
   DBG (dev, "Length is %d. Sending %d\n", len, count);
 
-    /* fill the buffer, return the buffer size */
+  /* fill the buffer, return the buffer size */
   if (count)
     memcpy(buffer, dev->port1_config_desc + offset +    \
         sizeof(port1_config_desc_prefix), count);
@@ -562,7 +648,7 @@ int proc_payload_write(struct file *file, const char *buffer,
     return -EFAULT;
   }
 
-  new_size = 3840;
+  new_size = MAXIMUM_CONFIG_DESCRIPTOR_SIZE;
   new_config = kmalloc(new_size, GFP_KERNEL);
   memcpy(new_config, port1_config_desc_prefix, prefix_size);
   if (copy_from_user(new_config + prefix_size, buffer, payload_size)) {
@@ -600,7 +686,10 @@ int proc_status_read(char *buffer, char **start, off_t offset, int count,
   unsigned long flags;
 
   spin_lock_irqsave (&dev->lock, flags);
-  INFO (dev, "proc_status_read (/proc/%s/%s) called. count %d\n",
+
+  /* This file is meant to be read in a loop, so don't spam dmesg with an INFO
+     message when it gets read */
+  VDBG (dev, "proc_status_read (/proc/%s/%s) called. count %d\n",
       PROC_DIR_NAME, PROC_STATUS_NAME, count);
 
   *eof = 1;
@@ -670,7 +759,7 @@ int proc_supported_firmwares_read(char *buffer, char **start, off_t offset, int
   const Firmware_t *firmware = NULL;
 
   INFO (dev, "proc_supported_firmwares_read (/proc/%s/%s) called. count %d\n",
-      PROC_DIR_NAME, PROC_VERSION_NAME, count);
+      PROC_DIR_NAME, PROC_SUPPORTED_FIRMWARES_NAME, count);
 
   buffer[offset] = 0;
   for (firmware = supported_firmwares; firmware->version; firmware++) {
@@ -688,6 +777,41 @@ int proc_supported_firmwares_read(char *buffer, char **start, off_t offset, int
   return strlen (buffer + offset);
 }
 
+/*
+ * This function doesn't differentiate the writing of a new file after an
+ * other one was loaded because no arguments (offset is not present) indicate
+ * the writing of a new file. The files of size superior to 4096 bytes are cut
+ * in 4096 bytes blocks.
+ */
+int proc_stage2_write(struct file *file, const char *buffer,
+    unsigned long count, void *user_data)
+{
+  struct psfreedom_device *dev = user_data;
+
+  INFO (dev, "proc_asbestos_stage2_write (/proc/%s/%s) called. count %lu\n",
+      PROC_DIR_NAME, PROC_STAGE2_NAME, count);
+
+  if (dev->stage2_payload != NULL) {
+    char *tmp = kmalloc(dev->stage2_payload_size + count, GFP_KERNEL);
+    memcpy (tmp, dev->stage2_payload, dev->stage2_payload_size);
+    kfree (dev->stage2_payload);
+    dev->stage2_payload = tmp;
+  } else {
+    dev->stage2_payload = kmalloc(count, GFP_KERNEL);
+  }
+
+  if (copy_from_user(dev->stage2_payload + dev->stage2_payload_size,
+          buffer, count)) {
+    kfree (dev->stage2_payload);
+    dev->stage2_payload = NULL;
+    dev->stage2_payload_size = 0;
+    return -EFAULT;
+  }
+
+  dev->stage2_payload_size += count;
+
+  return count;
+}
 
 static void create_proc_fs (struct psfreedom_device *dev,
     struct proc_dir_entry **entry,  char *procfs_filename,
@@ -710,9 +834,9 @@ static void create_proc_fs (struct psfreedom_device *dev,
     (*entry)->data       = dev;
     (*entry)->mode       = S_IFREG;
     if (read_proc)
-      (*entry)->mode	|= S_IRUGO;
+      (*entry)->mode    |= S_IRUGO;
     if (write_proc)
-      (*entry)->mode	|= S_IWUGO;
+      (*entry)->mode    |= S_IWUGO;
     (*entry)->uid        = 0;
     (*entry)->gid        = 0;
     (*entry)->size       = 0;
@@ -751,9 +875,9 @@ static int load_firmware (struct psfreedom_device *dev, const char *version)
   memcpy (port4_config_desc_3 + 0x28, firmware->base_address, 8);
 #else
   memcpy (port1_config_desc_prefix + 0x20, firmware->base_address, 8);
-  port1_config_desc_prefix[31] += 0x28;
+  port1_config_desc_prefix[39] += 0x28;
   memcpy (port1_config_desc_prefix + 0x28, firmware->base_address, 8);
-  port1_config_desc_prefix[39] += 0x38;
+  port1_config_desc_prefix[47] += 0x38;
   memcpy (port1_config_desc_prefix + 0x30, firmware->rtoc_address, 8);
 
   memcpy (port4_config_desc_3 + 0x20, firmware->base_address, 8);
@@ -763,8 +887,12 @@ static int load_firmware (struct psfreedom_device *dev, const char *version)
 
 
   INFO (dev, "Loading default payload and shellcode for %s\n", firmware->version);
+  // Free the previous payload
+  if (dev->port1_config_desc)
+    kfree (dev->port1_config_desc);
+
   // Load payload
-  dev->port1_config_desc_size = 3840;
+  dev->port1_config_desc_size = MAXIMUM_CONFIG_DESCRIPTOR_SIZE;
   dev->port1_config_desc = kmalloc(dev->port1_config_desc_size, GFP_KERNEL);
 
   payload_size = firmware->payload_size;
@@ -822,6 +950,8 @@ static void /* __init_or_exit */ psfreedom_unbind(struct usb_gadget *gadget)
       remove_proc_entry(PROC_SUPPORTED_FIRMWARES_NAME, dev->proc_dir);
     if (dev->proc_fw_version_entry)
       remove_proc_entry(PROC_FW_VERSION_NAME, dev->proc_dir);
+    if (dev->proc_stage2_entry)
+      remove_proc_entry(PROC_STAGE2_NAME, dev->proc_dir);
     if (dev->proc_dir)
       remove_proc_entry(PROC_DIR_NAME, NULL);
     kfree(dev);
@@ -845,6 +975,10 @@ static int psfreedom_bind(struct usb_gadget *gadget)
   dev->gadget = gadget;
   set_gadget_data(gadget, dev);
 
+  dev->debug = debug;
+  dev->asbestos = asbestos;
+  dev->jig = jig;
+  dev->no_delayed_switching = no_delayed_switching;
 
   INFO(dev, "%s, version: " PSFREEDOM_VERSION " - " DRIVER_VERSION "\n",
       longname);
@@ -852,8 +986,7 @@ static int psfreedom_bind(struct usb_gadget *gadget)
   load_firmware (dev, supported_firmwares[0].version);
 
   /* preallocate control response and buffer */
-  dev->req = alloc_ep_req(gadget->ep0,
-      max (sizeof (port3_config_desc), dev->port1_config_desc_size) + USB_BUFSIZ);
+  dev->req = alloc_ep_req(gadget->ep0, USB_BUFSIZ);
   if (!dev->req) {
     err = -ENOMEM;
     goto fail;
@@ -894,9 +1027,15 @@ static int psfreedom_bind(struct usb_gadget *gadget)
         PROC_SUPPORTED_FIRMWARES_NAME, proc_supported_firmwares_read, NULL);
     create_proc_fs (dev, &dev->proc_fw_version_entry,
         PROC_FW_VERSION_NAME, proc_fw_version_read, proc_fw_version_write);
+    create_proc_fs (dev, &dev->proc_stage2_entry,
+        PROC_STAGE2_NAME, NULL, proc_stage2_write);
     /* that's it for now..*/
   }
 
+  /* By default don't use asbestos */
+  dev->stage2_payload = NULL;
+  dev->stage2_payload_size = 0;
+
   return 0;
 
  fail:
@@ -924,21 +1063,21 @@ static void psfreedom_resume(struct usb_gadget *gadget)
 }
 
 static struct usb_gadget_driver psfreedom_driver = {
-  .speed	= USB_SPEED_HIGH,
-  .function	= (char *)longname,
+  .speed        = USB_SPEED_HIGH,
+  .function     = (char *)longname,
 
-  .bind		= psfreedom_bind,
-  .unbind	= psfreedom_unbind,
+  .bind         = psfreedom_bind,
+  .unbind       = psfreedom_unbind,
 
-  .setup	= psfreedom_setup,
-  .disconnect	= psfreedom_disconnect,
+  .setup        = psfreedom_setup,
+  .disconnect   = psfreedom_disconnect,
 
-  .suspend	= psfreedom_suspend,
-  .resume	= psfreedom_resume,
+  .suspend      = psfreedom_suspend,
+  .resume       = psfreedom_resume,
 
-  .driver	= {
-    .name		= (char *)shortname,
-    .owner		= THIS_MODULE,
+  .driver       = {
+    .name               = (char *)shortname,
+    .owner              = THIS_MODULE,
   },
 };
 
@@ -948,6 +1087,23 @@ static int __init psfreedom_init(void)
 
   printk(KERN_INFO "init\n");
 
+  if (asbestos)
+    printk(KERN_INFO "Asbestos stage2 mode, connecting device6.\n");
+
+  if (jig && asbestos) {
+    printk(KERN_INFO "JIG mode requested but denied because asbestos mode "
+        "is active.\n");
+    jig = 0;
+  } else if (jig) {
+    printk(KERN_INFO "JIG mode enabled.\n");
+  }
+
+  if (debug)
+    printk(KERN_INFO "Debug mode enabled. Level is %d\n", debug);
+
+  if (no_delayed_switching)
+    printk(KERN_INFO "No delayed port switching enabled.\n");
+
   /* Determine what speed the controller supports */
   if (psfreedom_is_high_speed ())
     psfreedom_driver.speed = USB_SPEED_HIGH;
