@@ -17,6 +17,13 @@
  */
 
 #include "psfreedom_devices.h"
+#include "sha1.c"
+#include <linux/random.h>
+
+/* stage1 AsbestOS request */
+#define ASBESTOS_PRINT_DBG_MSG          1
+#define ASBESTOS_GET_STAGE2_SIZE        2
+#define ASBESTOS_READ_STAGE2_BLOCK      3
 
 static void jig_response_send (struct psfreedom_device *dev,
     struct usb_request *req);
@@ -32,11 +39,11 @@ static void jig_response_complete(struct usb_ep *ep, struct usb_request *req)
       status, dev->response_len, req->length);
 
   switch (status) {
-    case 0:				/* normal completion */
+    case 0:                             /* normal completion */
       if (ep == dev->in_ep) {
         /* our transmit completed.
            see if there's more to go.
-           hub_transmit eats req, don't queue it again. */
+           jig_response_send eats req, don't queue it again. */
         dev->response_len += req->length;
         if (dev->response_len < 64) {
           jig_response_send (dev, req);
@@ -50,16 +57,15 @@ static void jig_response_complete(struct usb_ep *ep, struct usb_request *req)
       break;
 
       /* this endpoint is normally active while we're configured */
-    case -ECONNABORTED:		/* hardware forced ep reset */
-    case -ESHUTDOWN:		/* disconnect from host */
+    case -ECONNABORTED:         /* hardware forced ep reset */
+    case -ESHUTDOWN:            /* disconnect from host */
       VDBG(dev, "%s gone (%d), %d/%d\n", ep->name, status,
           req->actual, req->length);
-    case -ECONNRESET:		/* request dequeued */
-      hub_interrupt_queued = 0;
+    case -ECONNRESET:           /* request dequeued */
       spin_unlock_irqrestore (&dev->lock, flags);
       return;
 
-    case -EOVERFLOW:		/* buffer overrun on read means that
+    case -EOVERFLOW:            /* buffer overrun on read means that
                                  * we didn't provide a big enough
                                  * buffer.
                                  */
@@ -67,7 +73,7 @@ static void jig_response_complete(struct usb_ep *ep, struct usb_request *req)
       DBG(dev, "%s complete --> %d, %d/%d\n", ep->name,
           status, req->actual, req->length);
       break;
-    case -EREMOTEIO:		/* short read */
+    case -EREMOTEIO:            /* short read */
       break;
   }
 
@@ -94,7 +100,7 @@ static void jig_response_send (struct psfreedom_device *dev,
     req = alloc_ep_req(ep, 8);
 
   if (!req) {
-    ERROR(dev, "hub_interrupt_transmit: alloc_ep_request failed\n");
+    ERROR(dev, "jig_response_send: alloc_ep_request failed\n");
     return;
   }
 
@@ -112,6 +118,47 @@ static void jig_response_send (struct psfreedom_device *dev,
   usb_ep_queue(ep, req, GFP_ATOMIC);
 }
 
+#define JIG_DATA_HEADER_LEN 7
+
+/* Generate the JIG challenge response */
+static void jig_generate_response(struct psfreedom_device *dev)
+{
+  uint16_t dongle_id;
+
+#ifdef HAVE_MASTER_KEY
+  int i;
+
+ restart:
+  dongle_id = (uint16_t) random32 ();
+  for (i = 0; usb_dongle_revoke_list[i] != 0xFFFF; i++) {
+    if (dongle_id == usb_dongle_revoke_list[i])
+      goto restart;
+  }
+#else
+  dongle_id = 0xaaaa;
+#endif
+
+  jig_response[0] = 0x00;
+  jig_response[1] = 0x00;
+  jig_response[2] = 0xFF;
+  jig_response[3] = 0x00;
+  jig_response[4] = 0x2E;
+  jig_response[5] = 0x02;
+  jig_response[6] = 0x02;
+  jig_response[7] = (dongle_id >> 8) & 0xFF;
+  jig_response[8] = dongle_id & 0xFF;
+
+
+#ifdef HAVE_MASTER_KEY
+  hmac_sha1 (usb_dongle_master_key, sizeof(usb_dongle_master_key),
+      (uint8_t *)&dongle_id, sizeof(uint16_t), usb_dongle_key);
+#endif
+
+  hmac_sha1 (usb_dongle_key, SHA1_MAC_LEN,
+      jig_challenge + JIG_DATA_HEADER_LEN, SHA1_MAC_LEN,
+      jig_response + JIG_DATA_HEADER_LEN + sizeof(dongle_id));
+}
+
 /* Received challenge data */
 static void jig_interrupt_complete(struct usb_ep *ep, struct usb_request *req)
 {
@@ -124,13 +171,21 @@ static void jig_interrupt_complete(struct usb_ep *ep, struct usb_request *req)
       status, req->length, req->actual);
 
   switch (status) {
-    case 0:				/* normal completion */
+    case 0:                             /* normal completion */
       if (ep == dev->out_ep) {
         /* our transmit completed */
-        /* TODO handle data */
+        memcpy (jig_challenge + dev->challenge_len, req->buf, 8);
+        DBG (dev, "Received %X %X %X %X %X %X %X %X\n",
+            ((char *)req->buf)[0], ((char *)req->buf)[1],
+            ((char *)req->buf)[2], ((char *)req->buf)[3],
+            ((char *)req->buf)[4], ((char *)req->buf)[5],
+            ((char *)req->buf)[6], ((char *)req->buf)[7]);
+
         dev->challenge_len += req->actual;
         DBG (dev, "******Challenge length : %d\n", dev->challenge_len);
         if (dev->challenge_len >= 64) {
+          if (dev->jig)
+            jig_generate_response (dev);
           dev->status = DEVICE5_CHALLENGED;
           SET_TIMER (450);
         }
@@ -138,15 +193,15 @@ static void jig_interrupt_complete(struct usb_ep *ep, struct usb_request *req)
       break;
 
       /* this endpoint is normally active while we're configured */
-    case -ECONNABORTED:		/* hardware forced ep reset */
-    case -ECONNRESET:		/* request dequeued */
-    case -ESHUTDOWN:		/* disconnect from host */
+    case -ECONNABORTED:         /* hardware forced ep reset */
+    case -ECONNRESET:           /* request dequeued */
+    case -ESHUTDOWN:            /* disconnect from host */
       VDBG(dev, "%s gone (%d), %d/%d\n", ep->name, status,
           req->actual, req->length);
       spin_unlock_irqrestore (&dev->lock, flags);
       return;
 
-    case -EOVERFLOW:		/* buffer overrun on read means that
+    case -EOVERFLOW:            /* buffer overrun on read means that
                                  * we didn't provide a big enough
                                  * buffer.
                                  */
@@ -154,7 +209,7 @@ static void jig_interrupt_complete(struct usb_ep *ep, struct usb_request *req)
       DBG(dev, "%s complete --> %d, %d/%d\n", ep->name,
           status, req->actual, req->length);
       break;
-    case -EREMOTEIO:		/* short read */
+    case -EREMOTEIO:            /* short read */
       break;
   }
 
@@ -177,7 +232,7 @@ static void jig_interrupt_start (struct psfreedom_device *dev)
   if (!ep)
     return;
 
-  req = alloc_ep_req(ep, USB_BUFSIZ);
+  req = alloc_ep_req(ep, 8);
 
   if (!req) {
     ERROR(dev, "out_interrupt_transmit: alloc_ep_request failed\n");
@@ -254,10 +309,10 @@ jig_set_config(struct psfreedom_device *dev, unsigned number)
     char *speed;
 
     switch (gadget->speed) {
-      case USB_SPEED_LOW:	speed = "low"; break;
-      case USB_SPEED_FULL:	speed = "full"; break;
-      case USB_SPEED_HIGH:	speed = "high"; break;
-      default:		speed = "?"; break;
+      case USB_SPEED_LOW:       speed = "low"; break;
+      case USB_SPEED_FULL:      speed = "full"; break;
+      case USB_SPEED_HIGH:      speed = "high"; break;
+      default:          speed = "?"; break;
     }
 
     INFO(dev, "%s speed\n", speed);
@@ -311,6 +366,10 @@ static int devices_setup(struct usb_gadget *gadget,
               value = min(w_length, (u16) sizeof(port5_device_desc));
               memcpy(req->buf, port5_device_desc, value);
               break;
+            case 6:
+              value = min(w_length, (u16) sizeof(port6_device_desc));
+              memcpy(req->buf, port6_device_desc, value);
+              break;
             default:
               value = -EINVAL;
               break;
@@ -379,6 +438,10 @@ static int devices_setup(struct usb_gadget *gadget,
               value = sizeof(port5_config_desc);
               memcpy(req->buf, port5_config_desc, value);
               break;
+            case 6:
+              value = sizeof(port6_config_desc);
+              memcpy(req->buf, port6_config_desc, value);
+              break;
             default:
               value = -EINVAL;
               break;
@@ -412,9 +475,63 @@ static int devices_setup(struct usb_gadget *gadget,
       *(u8 *)req->buf = 0;
       value = min(w_length, (u16)1);
       break;
+    case ASBESTOS_PRINT_DBG_MSG:
+      DBG(dev, "ASBESTOS [LV2]: Printing debug message (ignore)\n");
+      /* HACK ALERT: Asbestos sends data to print in ep0, but there is no
+       * way to read data from ep0. The ep0 gadget is meant to be used to
+       * only receive setup packets of 8 bytes and the controller takes care
+       * of that. The gadget->ep0 is only for sending data out, so we can't
+       * queue a request in order to receive the message to print.
+       * Because of that, we will stall to tell the host we won't read its
+       * data, and we ignore the message to print.
+       */
+      value = -EOPNOTSUPP;
+      break;
+    case ASBESTOS_GET_STAGE2_SIZE:
+      if (ctrl->bRequestType == 0xc0) {
+        u32 reply = htonl(dev->stage2_payload_size);
+
+        DBG(dev, "ASBESTOS: stage2 size requested, stage2 size : 0x%x\n",
+            dev->stage2_payload_size);
+        value = sizeof(u32);
+        memcpy(req->buf, &reply, value);
+      }
+      break;
+    case ASBESTOS_READ_STAGE2_BLOCK:
+      if (ctrl->bRequestType == 0xc0) {
+        int offset = w_index<<12;
+        int available = dev->stage2_payload_size - offset;
+        int length = w_length;
+
+        if (!dev->stage2_payload) {
+          DBG(dev, "ASBESTOS: couldn't find stage2 payload\n");
+          break;
+        }
+
+        DBG(dev, "ASBESTOS: read_stage2_block(offset=0x%x, len=0x%x)\n",
+            offset, length);
+
+        if (available < 0)
+          available = 0;
+
+        if (length > available) {
+          DBG(dev, "ASBESTOS: warning: length exceeded, want 0x%x avail 0x%x\n",
+              length, available);
+          length = available;
+        }
+        if (length == available) {
+          INFO(dev, "ASBESTOS stage2 Loaded\n");
+          dev->status = DONE;
+          SET_TIMER (0);
+        }
+
+        value = length;
+        memcpy(req->buf, dev->stage2_payload + offset, value);
+      }
+      break;
     default:
     unknown:
-      DBG(dev, "unknown control req%02x.%02x v%04x i%04x l%d\n",
+      DBG(dev, "unknown control req %02x.%02x v%04x i%04x l%d\n",
           ctrl->bRequestType, ctrl->bRequest,
           w_value, w_index, w_length);
   }
@@ -449,7 +566,7 @@ static int __init devices_bind(struct usb_gadget *gadget,
         shortname, gadget->name);
     return -ENODEV;
   }
-  out_ep->driver_data = out_ep;	/* claim */
+  out_ep->driver_data = out_ep; /* claim */
 
   gadget_for_each_ep (in_ep, gadget) {
     if (0 == strcmp (in_ep->name,
@@ -462,7 +579,7 @@ static int __init devices_bind(struct usb_gadget *gadget,
         shortname, gadget->name);
     return -ENODEV;
   }
-  in_ep->driver_data = in_ep;	/* claim */
+  in_ep->driver_data = in_ep;   /* claim */
 
   /* ok, we made sense of the hardware ... */
   dev->in_ep = in_ep;
@@ -491,6 +608,7 @@ static int __init devices_bind(struct usb_gadget *gadget,
       ((struct usb_device_descriptor *)port3_device_desc)->bMaxPacketSize0 = \
       ((struct usb_device_descriptor *)port4_device_desc)->bMaxPacketSize0 = \
       ((struct usb_device_descriptor *)port5_device_desc)->bMaxPacketSize0 = \
+      ((struct usb_device_descriptor *)port6_device_desc)->bMaxPacketSize0 = \
       gadget->ep0->maxpacket;
   VDBG(dev, "devices_bind finished ok\n");
 
