@@ -23,6 +23,9 @@ private:
     Ui::AboutWindow *ui;
     QWebView *view;
     QWebViewSelectionSuppressor *suppressor;
+private slots:
+    void setProgressIndicatorOn();
+    void setProgressIndicatorOff();
 };
 
 #endif // ABOUTWINDOW_H
