@@ -24,28 +24,13 @@ class HelpWindow : public QMainWindow
 public:
     explicit HelpWindow(QWidget *parent = 0);
     ~HelpWindow();
-    QLabel *imageLabel;
     QWidget *centralwidget;
-    QString *PAGE1;
-    QString *PAGE2;
-    QString *PAGE3;
-    QString *PAGE4;
-    QString *PAGE5;
-    QLabel *imagePage1;
-    QLabel *imagePage2;
-    QLabel *imagePage3;
-    QLabel *imagePage4;
-    QLabel *imagePage5;
-    QPushButton *kernelMessagesButton;
     Ui::HelpWindow *ui;
 
 
 public slots:
-    void phat();
-    void slim();
     void showhelpSlim();
     void showhelpPhat();
-    void showKernelButton(int);
 
 private:
 
