@@ -5,6 +5,7 @@
 #include <QFile>
 #include <QString>
 #include <QTextStream>
+#include <QProcess>
 #ifdef Q_WS_MAEMO_5
 #include <QMaemo5InformationBox>
 #include <QPalette>
@@ -23,12 +24,13 @@ public:
     ~KernelMessagesWindow();
     Ui::KernelMessagesWindow *ui;
     QPalette *widget;
+    QStringList *dmesgList;
 
 public slots:
-    void showLogs();
+   // void showLogs();
 
 private:
-
+    QProcess *process;
 
 private slots:
     void exportlogs();
