@@ -10,9 +10,11 @@
 #include <kernelmessageswindow.h>
 #include <procwatch.h>
 #include <helpwindow.h>
+#include <QFile>
 #include <QPainter>
 #ifdef Q_WS_MAEMO_5
 #include <QMaemo5InformationBox>
+#include <QMaemo5ValueButton>
 #endif
 
 namespace Ui {
@@ -34,6 +36,15 @@ private:
     void connectSignals();
     QString m_BackgroundImage;
     void paintEvent(QPaintEvent*);
+    procwatch *myProcWatch;
+    QProcess *JBProcess;
+#ifdef Q_WS_MAEMO_5
+    QMaemo5ValueButton *jailbreakOnButton;
+    QMaemo5ValueButton *jailbreakOnHermesButton;
+#else
+    QPushButton *jailbreakOnButton;
+    QPushButton *jailbreakOnHermesButton;
+#endif
 
 private slots:
     void about();
