@@ -22,15 +22,13 @@ class procwatch : public QObject
 public:
     explicit procwatch(QObject *parent = 0);
     QTimer *timer;
-    QString *line;
+    QString *procStatus;
 
 signals:
 
 public slots:
-
     void detectStatus();
     void startTimer();
-    void stopTimer();
 
 private:
 
